import math
from microprediction.live.iex import iex_latest_prices
from microprediction.live.xraytickers import XRAY_TICKERS
from microprediction.live.xrayportfolios import XRAY_PORTFOLIO_NAMES



XRAY_NUM = len(XRAY_TICKERS)
XRAY_NAMES = ['r_' + str(k) + '.json' for k, ticker in enumerate(XRAY_TICKERS)]


def iex_scaled_log_xray(api_key:str):
    """
       Retrieve live prices
       api_key: An IEX api key
    """
    sp = iex_latest_prices(tickers=XRAY_TICKERS, api_key=api_key)
    return [1000 * math.log(v) for v in sp]


if __name__=='__main__':
    print(len(XRAY_NAMES))






from microprediction.live.xraytickers import XRAY_TICKERS
from microprediction.live.iex import iex_common_stock, iex_latest_prices
try:
    from credentials import IEX_KEY
except ImportError:
    raise EnvironmentError('You need a write key')
import numpy as np
import math
from getjson import getjson


def xray_portfolios():
    """ Retrieve xray portfolios """
    data = getjson('https://raw.githubusercontent.com/microprediction/microprediction/master/microprediction/live/xrayportfolios.json')
    return [ data[str(i)] for i in range(len(data)) ]

if False:
    XRAY_PORTFOLIOS = xray_portfolios()

NUM_PORTFOLIOS = 1500
XRAY_PORTFOLIO_NAMES = ['xray_' + str(i) for i in range(NUM_PORTFOLIOS)]


def normalize(w):
    return [ wi/sum(w) for wi in w]


def create_xray_portfolios():
    # One-off determination of portfolio weights, as perturbations of pseudo-index
    common = iex_common_stock(tickers=XRAY_TICKERS, api_key=IEX_KEY)
    print(common)
    prices = iex_latest_prices(tickers=XRAY_TICKERS, api_key=IEX_KEY)
    index_like = normalize([price * common for price, common in zip(prices, common)])
    XRAY_PORTFOLIOS = [normalize([wi * math.exp(np.random.randn()) for wi in index_like]) for _ in
                       range(NUM_PORTFOLIOS)]
    XRAY_PORTFOLIOS_DICT = dict([(k, p) for k, p in enumerate(XRAY_PORTFOLIOS)])

    from microprediction.whereami import TOP
    import os
    import json
    XRAY_COMMON_JSON = os.path.join(TOP, 'live', 'xrayportfolios.json')
    with open(XRAY_COMMON_JSON, 'wt') as fp:
        json.dump(obj=XRAY_PORTFOLIOS_DICT, fp=fp)


if __name__=='__main__':
    CREATING = False
    if CREATING:
        # DON'T DO THIS ... SUPPOSED TO BE ONE-OFF
        create_xray_portfolios()
    else:
        portfolios = xray_portfolios()
        print(len(portfolios))




# IEX stock prices
# You can get your api_key at https://iexcloud.io/
# See https://iexcloud.io/docs/api/#quote
from getjson import getjson


def iex_latest_prices(tickers, api_key:str)->[float]:
    """
        Grab multiple stock prices
        IEX allows only 100 at a time
    """
    if len(tickers)>100:
        tickerlists = [tickers[i:i + 100] for i in range(0, len(tickers), 100)]
        mids = [ iex_latest_prices(tickers=ticks, api_key=api_key) for ticks in tickerlists ]
        mids_flat = [item for sublist in mids for item in sublist]
        return mids_flat
    else:
        symbols = ','.join(tickers)
        paid_url = 'https://cloud.iexapis.com/v1/stock/market/batch?types=price&symbols='+symbols+'&token=' + api_key
        print(paid_url)
        data = getjson(paid_url)
        mids = [ data[ticker.upper()]['price'] for ticker in tickers]
        return mids


def iex_common_stock(tickers, api_key:str)->[float]:
    import time
    common = list()
    missing = list()
    for ticker_no, ticker in enumerate(tickers):
        if ticker_no % 100 ==0:
            print(ticker_no)
        paid_url = 'https://cloud.iexapis.com/v1/stock/'+ticker+'/balance-sheet?token=' + api_key
        data = getjson(paid_url, paid_url)
        try:
            cm = data['balancesheet'][0]['commonStock']
        except:
            cm = 0
            missing.append(ticker)
        common.append(cm)
        time.sleep(0.1)
    print('The following are missing balance sheet information ')
    print(missing)
    return common



if __name__ == '__main__':
    from pprint import pprint
    from getjson import getjson
    try:
        from credentials import IEX_KEY
    except ImportError:
        raise EnvironmentError('you need a write key')
    tickers = ['aapl','googl']
    data = iex_latest_prices(tickers=tickers, api_key=IEX_KEY)
    pprint(data)
    common = iex_common_stock(tickers=tickers, api_key=IEX_KEY)
    print(common)